@@ -1,5 +1,3 @@
-## $Id:readBrukerFlexData.R 381 2011-02-15 15:58:49Z sgibb $
-##
 ## Copyright 2010-2012 Sebastian Gibb
 ## <mail@sebastiangibb.de>
 ##
@@ -51,13 +49,13 @@ readBrukerFlexDir <- function(brukerFlexDir, removeCalibrationScans=TRUE,
         removeMetaData=FALSE, useHpc=TRUE, useSpectraNames=TRUE,
         filterZeroIntensities=FALSE,
         verbose=FALSE) {
-    if (verbose)
+    if (verbose) {
         message("Look for spectra in ", sQuote(brukerFlexDir), " ...");
+    }
 
     if ((!file.exists(brukerFlexDir)) || (!file.info(brukerFlexDir)$isdir)) {
-        warning("Directory ", sQuote(brukerFlexDir), 
-                " doesn't exists or is no directory!");
-        return(NA);
+        stop("Directory ", sQuote(brukerFlexDir), 
+             " doesn't exists or is no directory!");
     }
 
     ## look for fid files (alphabetical sort)
@@ -72,8 +70,7 @@ readBrukerFlexDir <- function(brukerFlexDir, removeCalibrationScans=TRUE,
     }
 
     if (length(files) <= 0) {
-        warning("Directory doesn't contain any fid file.");
-        return(NA);
+        stop("Directory doesn't contain any fid file.");
     }
 
     ## resort files in correct order, using gtools package
@@ -97,8 +94,9 @@ readBrukerFlexDir <- function(brukerFlexDir, removeCalibrationScans=TRUE,
 
     if (!removeMetaData & useSpectraNames) {
         ## rewrite names if metadata exists
-        if (verbose)
+        if (verbose) {
             message("look for spectra names ...");
+        }
 
         names(brukerFlexData) <- sapply(X=brukerFlexData, FUN=function(x) {
                 if (!is.null(x$metaData$sampleName)) {
@@ -133,22 +131,21 @@ readBrukerFlexDir <- function(brukerFlexDir, removeCalibrationScans=TRUE,
 ##
 readBrukerFlexFile <- function(fidFile, removeMetaData=FALSE, useHpc=TRUE,
                                filterZeroIntensities=FALSE, verbose=FALSE) {
-    ## try to get absolute file path
-    fidFile <- normalizePath(fidFile);
-
-    if (verbose)
+    if (verbose) {
         message("Reading spectrum from ", sQuote(fidFile), " ...");
+    }
   
     if (!file.exists(fidFile)) {
-        warning("File ", sQuote(fidFile), " doesn't exists!");
-        return(NA);
+        stop("File ", sQuote(fidFile), " doesn't exists!");
     }
 
     if (file.info(fidFile)$isdir) {
-        warning("Not a fid file! ", sQuote(fidFile), " is a directory.");
-        return(NA);
+        stop("Not a fid file! ", sQuote(fidFile), " is a directory.");
     }
 
+    ## try to get absolute file path
+    fidFile <- normalizePath(fidFile);
+
     ## read metadata
     metaData <- .readAcquFile(fidFile=fidFile, verbose=verbose);
 
@@ -156,7 +153,8 @@ readBrukerFlexFile <- function(fidFile, removeMetaData=FALSE, useHpc=TRUE,
     intensity <- .readFidFile(fidFile, metaData$number, metaData$byteOrder);
 
     ## calculate tof of metadata
-    tof <- metaData$timeDelay + ((0:(metaData$number-1)) * metaData$timeDelta);
+    tof <- as.double(metaData$timeDelay + 
+                     ((0:(metaData$number-1)) * metaData$timeDelta));
 
     ## remove times which have no intensity (intensity == 0), e.g. generated by
     ## REFLECTOR mode
@@ -379,12 +377,12 @@ readBrukerFlexFile <- function(fidFile, removeMetaData=FALSE, useHpc=TRUE,
 .readAcquFile <- function(fidFile, verbose=FALSE) {
     acquFile <- sub(pattern="fid$", x=fidFile, replacement="acqu");
 
-    if (verbose)
+    if (verbose) {
         message("Reading metadata from ", sQuote(acquFile), " ...");
+    }
 
     if (!file.exists(acquFile)) {
-        warning("File ", sQuote(acquFile), " doesn't exists!");
-        return(NA);
+        stop("File ", sQuote(acquFile), " doesn't exists!");
     }
 
     con <- file(acquFile, "rt");
@@ -403,16 +401,16 @@ readBrukerFlexFile <- function(fidFile, removeMetaData=FALSE, useHpc=TRUE,
     metaData$timeDelay <- as.double(.grepAcquValue("##\\$DELAY=", acquLines));
     metaData$timeDelta <- as.double(.grepAcquValue("##\\$DW=", acquLines));
     metaData$calibrationConstants <- c(
-        c1=as.double(.grepAcquValue("##\\$ML1=", acquLines)), 
-        c2=as.double(.grepAcquValue("##\\$ML2=", acquLines)),
-        c3=as.double(.grepAcquValue("##\\$ML3=", acquLines)));
+        c1=.grepAcquDoubleValue("##\\$ML1=", acquLines), 
+        c2=.grepAcquDoubleValue("##\\$ML2=", acquLines),
+        c3=.grepAcquDoubleValue("##\\$ML3=", acquLines));
 
     ## obligate HPC
     metaData$hpcLimits <- c(
-        minMass=as.double(.grepAcquValue("##\\$HPClBLo=", acquLines)),
-        maxMass=as.double(.grepAcquValue("##\\$HPClBHi=", acquLines)));
-    metaData$hpcOrder <- as.double(.grepAcquValue("##\\$HPClOrd=", 
-        acquLines));
+        minMass=.grepAcquDoubleValue("##\\$HPClBLo=", acquLines),
+        maxMass=.grepAcquDoubleValue("##\\$HPClBHi=", acquLines));
+    metaData$hpcOrder <-
+        as.double(.grepAcquValue("##\\$HPClOrd=", acquLines));
     metaData$hpcUse <- 
         as.logical(.grepAcquValue("##\\$HPClUse=", acquLines) == "yes");
   
@@ -461,7 +459,7 @@ readBrukerFlexFile <- function(fidFile, removeMetaData=FALSE, useHpc=TRUE,
     metaData$acquisitionOperatorMode <- metaData$tofMode;
 
     metaData$laserAttenuation <- 
-        as.double(.grepAcquValue("##\\$ATTEN=", acquLines));
+        .grepAcquDoubleValue("##\\$ATTEN=", acquLines);
 
     metaData$comments <- c(.grepAcquValue("##\\$CMT1=", acquLines),
                            .grepAcquValue("##\\$CMT2=", acquLines),
@@ -470,12 +468,11 @@ readBrukerFlexFile <- function(fidFile, removeMetaData=FALSE, useHpc=TRUE,
 
     metaData$deflection <- 
         as.logical(.grepAcquValue("##\\$DEFLON=", acquLines) == "yes");
-    metaData$digitizerType <-
-        as.double(.grepAcquValue("##\\$DIGTYP=", acquLines));
+    metaData$digitizerType <- .grepAcquDoubleValue("##\\$DIGTYP=", acquLines);
     metaData$deflectionPulserCal1 <- 
-        as.double(.grepAcquValue("##\\$DPCAL1=", acquLines));
+        .grepAcquDoubleValue("##\\$DPCAL1=", acquLines);
     metaData$deflectionPulserMass <- 
-        as.double(.grepAcquValue("##\\$DPMASS=", acquLines));
+        .grepAcquDoubleValue("##\\$DPMASS=", acquLines);
     metaData$flexControlVersion <- .grepAcquValue("##\\$FCVer=", acquLines);
     metaData$id <- .grepAcquValue("##\\$ID_raw=", acquLines);
 
@@ -483,10 +480,9 @@ readBrukerFlexFile <- function(fidFile, removeMetaData=FALSE, useHpc=TRUE,
     metaData$instrumentId <- .grepAcquValue("##\\$InstrID=", acquLines);
     metaData$instrumentType <- .grepAcquValue("##\\$InstTyp=", acquLines);
 
-    metaData$massError <- 
-        as.double(.grepAcquValue("##\\$Masserr=", acquLines));
+    metaData$massError <- .grepAcquDoubleValue("##\\$Masserr=", acquLines);
 
-    metaData$laserShots <- 
+    metaData$laserShots <-
         as.double(.grepAcquValue("##\\$NoSHOTS=", acquLines));
 
     if (metaData$laserShots == 0) {
@@ -496,8 +492,7 @@ readBrukerFlexFile <- function(fidFile, removeMetaData=FALSE, useHpc=TRUE,
 
     metaData$patch <- .grepAcquValue("##\\$PATCHNO=", acquLines);
     metaData$path <- .grepAcquValue("##\\$PATH=", acquLines);
-    metaData$laserRepetition <-
-        as.double(.grepAcquValue("##\\$REPHZ=", acquLines));
+    metaData$laserRepetition <- .grepAcquDoubleValue("##\\$REPHZ=", acquLines);
     metaData$spot <- .grepAcquValue("##\\$SPOTNO=", acquLines);
     
     sptype <- as.double(.grepAcquValue("##\\$SPType=", acquLines));
@@ -509,7 +504,7 @@ readBrukerFlexFile <- function(fidFile, removeMetaData=FALSE, useHpc=TRUE,
             metaData$spectrumType <- sptype;
         });
 
-    metaData$targetCount <- 
+    metaData$targetCount <-
         as.double(.grepAcquValue("##\\$TgCount", acquLines));
     metaData$targetIdString <- .grepAcquValue("##\\$TgIDS", acquLines);
     metaData$targetSerialNumber <- .grepAcquValue("##\\$TgSer", acquLines);
@@ -546,8 +541,7 @@ readBrukerFlexFile <- function(fidFile, removeMetaData=FALSE, useHpc=TRUE,
 ##
 .readFidFile <- function(fidFile, nIntensities, endian="little") {
     if (!file.exists(fidFile)) {
-        warning("File ", sQuote(fidFile), " doesn't exists!");
-        return(NA);
+        stop("File ", sQuote(fidFile), " doesn't exists!");
     }
 
     con <- file(fidFile, "rb");
@@ -586,6 +580,25 @@ readBrukerFlexFile <- function(fidFile, removeMetaData=FALSE, useHpc=TRUE,
     return(tmpLine);
 }
 
+## function .grepAcquDoubleValue
+##  helper function to extract double values from acqu file
+##
+## params:
+##  patternStr: pattern to look for
+##  srcStr: where to look for patternStr
+##
+## returns:
+##  double value of the value given in patternStr
+##
+.grepAcquDoubleValue <- function(patternStr, srcStr) {
+    strValue <- .grepAcquValue(patternStr, srcStr);
+
+    ## replace comma by dot
+    strValue <- gsub(",", replacement=".", strValue);
+
+    return(as.double(strValue));
+}
+
 ## function .sampleName
 ##  guess name of current spot from filename
 ##
